@@ -8,7 +8,6 @@
 // PURPOSE.
 
 #include <cstdlib>              // for strtoul()
-#include <iostream>
 #include <string>
 #include <vector>
 
@@ -40,7 +39,8 @@ app_state::app_state()
     keys(this), work(db, lua),
 //    search_root(current_root_path()),
 //    diff_format(unified_diff),
-    branch_is_sticky(false)
+    branch_is_sticky(false),
+    project(*this)
 {
   db.set_app(this);
   lua.set_app(this);
@@ -77,7 +77,10 @@ app_state::allow_workspace()
 void
 app_state::process_options()
 {
-  utf8 database_option, branch_option, key_option, keydir_option;
+  system_path database_option;
+  branch_name branch_option;
+  rsa_keypair_id key_option;
+  system_path keydir_option;
 
   if (!found_workspace)
     return;
@@ -87,37 +90,41 @@ app_state::process_options()
                       key_option, keydir_option);
 
   // Workspace options are not to override the command line.
-  if (db.get_filename().as_internal().empty() && !database_option().empty())
-    db.set_filename(system_path(database_option));
+  if (db.get_filename().as_internal().empty() && !database_option.as_internal().empty())
+    db.set_filename(database_option);
 
-  if (keys.get_key_dir().as_internal().empty() && !keydir_option().empty())
-    set_key_dir(system_path(keydir_option));
+  if (keys.get_key_dir().as_internal().empty() && !keydir_option.as_internal().empty())
+    set_key_dir(keydir_option);
 
-  if (opts.branch_name().empty() && !branch_option().empty())
+  if (opts.branchname().empty() && !branch_option().empty())
     {
-      opts.branch_name = branch_option;
+      opts.branchname = branch_option;
       branch_is_sticky = true;
     }
 
-  L(FL("branch name is '%s'") % opts.branch_name);
+  L(FL("branch name is '%s'") % opts.branchname);
 
   if (!opts.key_given)
-    internalize_rsa_keypair_id(key_option, opts.signing_key);
+    opts.signing_key = key_option;
 }
 
 void
 app_state::write_options()
 {
-  utf8 database_option, branch_option, key_option, keydir_option;
+  system_path database_option;
+  branch_name branch_option;
+  rsa_keypair_id key_option;
+  system_path keydir_option;
 
-  database_option = db.get_filename().as_internal();
-  keydir_option = keys.get_key_dir().as_internal();
+  database_option = db.get_filename();
+  keydir_option = keys.get_key_dir();
 
   if (branch_is_sticky)
-    branch_option = opts.branch_name;
+    branch_option = opts.branchname;
 
   if (opts.key_given)
-    externalize_rsa_keypair_id(opts.signing_key, key_option);
+    key_option = opts.signing_key;
+
   work.set_ws_options(database_option, branch_option,
                       key_option, keydir_option);
 }
@@ -140,6 +147,7 @@ app_state::create_workspace(system_path const & new_dir)
 
   mkdir_p(new_dir);
   go_to_workspace(new_dir);
+  mark_std_paths_used();
 
   N(!directory_exists(bookkeeping_root),
     F("monotone bookkeeping directory '%s' already exists in '%s'")
@@ -160,6 +168,18 @@ app_state::create_workspace(system_path const & new_dir)
   if (lua.hook_use_inodeprints())
     work.enable_inodeprints();
 
+  found_workspace = true;
+  if (global_sanity.filename.empty())
+    {
+      bookkeeping_path dump_path;
+      work.get_local_dump_path(dump_path);
+      L(FL("setting dump path to %s") % dump_path);
+      // The 'true' means that, e.g., if we're running checkout,
+      // then it's okay for dumps to go into our starting working
+      // dir's _MTN rather than the new workspace dir's _MTN.
+      global_sanity.filename = system_path(dump_path, false).as_external();
+    }
+
   load_rcfiles();
 }
 
@@ -167,7 +187,20 @@ void
 app_state::set_database(system_path const & filename)
 {
   if (!filename.empty())
-    db.set_filename(filename);
+    {
+      db.set_filename(filename);
+      
+      if (found_workspace)
+        {
+          system_path database_option(filename);
+          branch_name branch_option;
+          rsa_keypair_id key_option;
+          system_path keydir_option;
+      
+          work.set_ws_options(database_option, branch_option,
+                      key_option, keydir_option);
+        }
+    }
 }
 
 void
@@ -193,6 +226,12 @@ app_state::make_branch_sticky()
     }
 }
 
+project_t &
+app_state::get_project()
+{
+  return project;
+}
+
 void
 app_state::set_root(system_path const & path)
 {
