@@ -1,9 +1,9 @@
 /*************************************************
 * Default Engine Algorithms Source File          *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
-#include <botan/def_eng.h>
+#include <botan/eng_def.h>
 #include <botan/lookup.h>
 #include <botan/parsing.h>
 
@@ -18,6 +18,9 @@
 
 #include <botan/hmac.h>
 
+#include <botan/mode_pad.h>
+#include <botan/pkcs5.h>
+
 namespace Botan {
 
 /*************************************************
@@ -68,7 +71,7 @@ BlockCipher*
 Default_Engine::find_block_cipher(const std::string& algo_spec) const
    {
    std::vector<std::string> name = parse_algorithm_name(algo_spec);
-   if(name.size() == 0)
+   if(name.empty())
       return 0;
    const std::string algo_name = deref_alias(name[0]);
 
@@ -90,14 +93,13 @@ StreamCipher*
 Default_Engine::find_stream_cipher(const std::string& algo_spec) const
    {
    std::vector<std::string> name = parse_algorithm_name(algo_spec);
-   if(name.size() == 0)
+   if(name.empty())
       return 0;
    const std::string algo_name = deref_alias(name[0]);
 
    HANDLE_TYPE_ONE_U32BIT("ARC4", ARC4, 0);
    HANDLE_TYPE_ONE_U32BIT("RC4_drop", ARC4, 768);
 
-
    return 0;
    }
 
@@ -108,7 +110,7 @@ HashFunction*
 Default_Engine::find_hash(const std::string& algo_spec) const
    {
    std::vector<std::string> name = parse_algorithm_name(algo_spec);
-   if(name.size() == 0)
+   if(name.empty())
       return 0;
    const std::string algo_name = deref_alias(name[0]);
 
@@ -125,7 +127,7 @@ MessageAuthenticationCode*
 Default_Engine::find_mac(const std::string& algo_spec) const
    {
    std::vector<std::string> name = parse_algorithm_name(algo_spec);
-   if(name.size() == 0)
+   if(name.empty())
       return 0;
    const std::string algo_name = deref_alias(name[0]);
 
@@ -134,4 +136,41 @@ Default_Engine::find_mac(const std::string& algo_spec) const
    return 0;
    }
 
+/*************************************************
+* Look for an algorithm with this name           *
+*************************************************/
+S2K* Default_Engine::find_s2k(const std::string& algo_spec) const
+   {
+   std::vector<std::string> name = parse_algorithm_name(algo_spec);
+   if(name.empty())
+      return 0;
+
+   const std::string algo_name = deref_alias(name[0]);
+
+   HANDLE_TYPE_ONE_STRING("PBKDF1", PKCS5_PBKDF1);
+   HANDLE_TYPE_ONE_STRING("PBKDF2", PKCS5_PBKDF2);
+
+   return 0;
+   }
+
+/*************************************************
+* Look for an algorithm with this name           *
+*************************************************/
+BlockCipherModePaddingMethod*
+Default_Engine::find_bc_pad(const std::string& algo_spec) const
+   {
+   std::vector<std::string> name = parse_algorithm_name(algo_spec);
+   if(name.empty())
+      return 0;
+
+   const std::string algo_name = deref_alias(name[0]);
+
+   HANDLE_TYPE_NO_ARGS("PKCS7",       PKCS7_Padding);
+   HANDLE_TYPE_NO_ARGS("OneAndZeros", OneAndZeros_Padding);
+   HANDLE_TYPE_NO_ARGS("X9.23",       ANSI_X923_Padding);
+   HANDLE_TYPE_NO_ARGS("NoPadding",   Null_Padding);
+
+   return 0;
+   }
+
 }
