@@ -1,6 +1,6 @@
 /*************************************************
 * Global RNG Header File                         *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #ifndef BOTAN_GLOBAL_RNG_H__
@@ -15,8 +15,8 @@ namespace Botan {
 *************************************************/
 namespace Global_RNG {
 
-void randomize(byte[], u32bit, RNG_Quality = SessionKey);
-byte random(RNG_Quality = SessionKey);
+void randomize(byte[], u32bit);
+byte random();
 
 void add_entropy(const byte[], u32bit);
 void add_entropy(EntropySource&, bool = true);
@@ -27,17 +27,6 @@ void add_es(EntropySource*, bool = true);
 
 }
 
-/*************************************************
-* RNG Control Functions                          *
-*************************************************/
-namespace Init {
-
-void set_global_rngs(RandomNumberGenerator*, RandomNumberGenerator*);
-void init_rng_subsystem();
-void shutdown_rng_subsystem();
-
-}
-
 }
 
 #endif
