@@ -12,8 +12,6 @@
 #include <string>
 #include <vector>
 
-#include <boost/date_time/gregorian/gregorian.hpp>
-#include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/tuple/tuple.hpp>
@@ -224,6 +222,11 @@ erase_bogus_certs(vector< revision<cert> > & certs,
 cert::cert()
 {}
 
+cert::cert(std::string const & s)
+{
+  read_cert(s, *this);
+}
+
 cert::cert(hexenc<id> const & ident,
            cert_name const & name,
            base64<cert_value> const & value,
@@ -269,12 +272,12 @@ void
 read_cert(string const & in, cert & t)
 {
   size_t pos = 0;
-  id hash = extract_substring(in, pos,
-                              constants::merkle_hash_length_in_bytes,
-                              "cert hash");
-  id ident = extract_substring(in, pos,
-                               constants::merkle_hash_length_in_bytes,
-                               "cert ident");
+  id hash = id(extract_substring(in, pos,
+                                 constants::merkle_hash_length_in_bytes,
+                                 "cert hash"));
+  id ident = id(extract_substring(in, pos,
+                                  constants::merkle_hash_length_in_bytes,
+                                  "cert ident"));
   string name, val, key, sig;
   extract_variable_length_string(in, name, pos, "cert name");
   extract_variable_length_string(in, val, pos, "cert val");
@@ -440,7 +443,7 @@ check_cert(app_state & app, cert const & t)
 
 // "special certs"
 
-string const branch_cert_name("branch");
+cert_name const branch_cert_name("branch");
 
 void
 get_user_key(rsa_keypair_id & key, app_state & app)
@@ -452,12 +455,8 @@ get_user_key(rsa_keypair_id & key, app_state & app)
       return;
     }
 
-  if (app.opts.branch_name() != "")
-    {
-      cert_value branch(app.opts.branch_name());
-      if (app.lua.hook_get_branch_key(branch, key))
-        return;
-    }
+  if (app.lua.hook_get_branch_key(app.opts.branchname, key))
+    return;
 
   vector<rsa_keypair_id> all_privkeys;
   app.keys.get_keys(all_privkeys);
@@ -470,39 +469,46 @@ get_user_key(rsa_keypair_id & key, app_state & app)
   key = all_privkeys[0];
 }
 
+// Guess which branch is appropriate for a commit below IDENT.
+// APP may override.  Branch name is returned in BRANCHNAME.
+// Does not modify branch state in APP.
 void
-guess_branch(revision_id const & ident,
-             app_state & app,
-             cert_value & branchname)
+guess_branch(revision_id const & ident, app_state & app, branch_name & branchname)
 {
-  if ((app.opts.branch_name() != "") && app.opts.branch_given)
-    {
-      branchname = app.opts.branch_name();
-    }
+  if (app.opts.branch_given && !app.opts.branchname().empty())
+    branchname = app.opts.branchname;
   else
     {
       N(!ident.inner()().empty(),
         F("no branch found for empty revision, "
           "please provide a branch name"));
 
-      vector< revision<cert> > certs;
-      cert_name branch(branch_cert_name);
-      app.db.get_revision_certs(ident, branch, certs);
-      erase_bogus_certs(certs, app);
+      set<branch_name> branches;
+      app.get_project().get_revision_branches(ident, branches);
 
-      N(certs.size() != 0,
+      N(branches.size() != 0,
         F("no branch certs found for revision %s, "
           "please provide a branch name") % ident);
 
-      N(certs.size() == 1,
+      N(branches.size() == 1,
         F("multiple branch certs found for revision %s, "
           "please provide a branch name") % ident);
 
-      decode_base64(certs[0].inner().value, branchname);
-      app.opts.branch_name = branchname();
+      set<branch_name>::iterator i = branches.begin();
+      I(i != branches.end());
+      branchname = *i;
     }
 }
 
+// As above, but set the branch name in the app state.
+void
+guess_branch(revision_id const & ident, app_state & app)
+{
+  branch_name branchname;
+  guess_branch(ident, app, branchname);
+  app.opts.branchname = branchname;
+}
+
 void
 make_simple_cert(hexenc<id> const & id,
                  cert_name const & nm,
@@ -519,7 +525,7 @@ make_simple_cert(hexenc<id> const & id,
   c = t;
 }
 
-static void
+void
 put_simple_revision_cert(revision_id const & id,
                          cert_name const & nm,
                          cert_value const & val,
@@ -542,87 +548,25 @@ cert_revision_in_branch(revision_id const & rev,
                             branchname, app, pc);
 }
 
-namespace
-{
-  struct not_in_branch : public is_failure
-  {
-    app_state & app;
-    base64<cert_value > const & branch_encoded;
-    not_in_branch(app_state & app,
-                  base64<cert_value> const & branch_encoded)
-      : app(app), branch_encoded(branch_encoded)
-    {}
-    virtual bool operator()(revision_id const & rid)
-    {
-      vector< revision<cert> > certs;
-      app.db.get_revision_certs(rid,
-                                cert_name(branch_cert_name),
-                                branch_encoded,
-                                certs);
-      erase_bogus_certs(certs, app);
-      return certs.empty();
-    }
-  };
-}
-
-void
-get_branch_heads(cert_value const & branchname,
-                 app_state & app,
-                 set<revision_id> & heads)
-{
-  L(FL("getting heads of branch %s") % branchname);
-  base64<cert_value> branch_encoded;
-  encode_base64(branchname, branch_encoded);
-
-  app.db.get_revisions_with_cert(cert_name(branch_cert_name),
-                                 branch_encoded,
-                                 heads);
-
-  not_in_branch p(app, branch_encoded);
-  erase_ancestors_and_failures(heads, p, app);
-  L(FL("found heads of branch %s (%s heads)") % branchname % heads.size());
-}
-
 
 // "standard certs"
 
-string const date_cert_name = "date";
-string const author_cert_name = "author";
-string const tag_cert_name = "tag";
-string const changelog_cert_name = "changelog";
-string const comment_cert_name = "comment";
-string const testresult_cert_name = "testresult";
+cert_name const date_cert_name("date");
+cert_name const author_cert_name("author");
+cert_name const tag_cert_name("tag");
+cert_name const changelog_cert_name("changelog");
+cert_name const comment_cert_name("comment");
+cert_name const testresult_cert_name("testresult");
 
 
 void
 cert_revision_date_time(revision_id const & m,
-                        boost::posix_time::ptime t,
+                        date_t const & t,
                         app_state & app,
                         packet_consumer & pc)
 {
-  string val = boost::posix_time::to_iso_extended_string(t);
-  put_simple_revision_cert(m, date_cert_name, val, app, pc);
-}
-
-void
-cert_revision_date_time(revision_id const & m,
-                        time_t t,
-                        app_state & app,
-                        packet_consumer & pc)
-{
-  // make sure you do all your CVS conversions by 2038!
-  boost::posix_time::ptime tmp(boost::gregorian::date(1970,1,1),
-                               boost::posix_time::seconds(static_cast<long>(t)));
-  cert_revision_date_time(m, tmp, app, pc);
-}
-
-void
-cert_revision_date_now(revision_id const & m,
-                       app_state & app,
-                       packet_consumer & pc)
-{
-  cert_revision_date_time
-    (m, boost::posix_time::second_clock::universal_time(), app, pc);
+  cert_value val = cert_value(t.as_iso_8601_extended());
+  put_simple_revision_cert(m, cert_name(date_cert_name), val, app, pc);
 }
 
 void
@@ -632,7 +576,7 @@ cert_revision_author(revision_id const & m,
                      packet_consumer & pc)
 {
   put_simple_revision_cert(m, author_cert_name,
-                           author, app, pc);
+                           cert_value(author), app, pc);
 }
 
 void
@@ -641,11 +585,12 @@ cert_revision_author_default(revision_id const & m,
                              packet_consumer & pc)
 {
   string author;
-  if (!app.lua.hook_get_author(app.opts.branch_name(), author))
+  rsa_keypair_id key;
+  get_user_key(key, app);
+
+  if (!app.lua.hook_get_author(app.opts.branchname, key, author))
     {
-      rsa_keypair_id key;
-      get_user_key(key, app),
-        author = key();
+      author = key();
     }
   cert_revision_author(m, author, app, pc);
 }
@@ -657,7 +602,7 @@ cert_revision_tag(revision_id const & m,
                   packet_consumer & pc)
 {
   put_simple_revision_cert(m, tag_cert_name,
-                           tagname, app, pc);
+                           cert_value(tagname), app, pc);
 }
 
 
@@ -668,7 +613,7 @@ cert_revision_changelog(revision_id const & m,
                         packet_consumer & pc)
 {
   put_simple_revision_cert(m, changelog_cert_name,
-                           changelog(), app, pc);
+                           cert_value(changelog()), app, pc);
 }
 
 void
@@ -678,7 +623,7 @@ cert_revision_comment(revision_id const & m,
                       packet_consumer & pc)
 {
   put_simple_revision_cert(m, comment_cert_name,
-                           comment(), app, pc);
+                           cert_value(comment()), app, pc);
 }
 
 void
@@ -703,7 +648,7 @@ cert_revision_testresult(revision_id const & r,
                               "tried '0/1' 'yes/no', 'true/false', "
                               "'pass/fail'");
 
-  put_simple_revision_cert(r, testresult_cert_name, lexical_cast<string>(passed), app, pc);
+  put_simple_revision_cert(r, testresult_cert_name, cert_value(lexical_cast<string>(passed)), app, pc);
 }
 
 // Local Variables:
