@@ -40,17 +40,17 @@ void
 charset_convert(string const & src_charset,
                 string const & dst_charset,
                 string const & src,
-                string & dst)
+                string & dst,
+                bool best_effort)
 {
   if (src_charset == dst_charset)
     dst = src;
   else
     {
-      L(FL("converting %d bytes from %s to %s") % src.size()
-        % src_charset % dst_charset);
       char * converted = stringprep_convert(src.c_str(),
                                             dst_charset.c_str(),
-                                            src_charset.c_str());
+                                            src_charset.c_str(),
+                                            best_effort);
       E(converted != NULL,
         F("failed to convert string from %s to %s: '%s'")
          % src_charset % dst_charset % src);
@@ -59,15 +59,6 @@ charset_convert(string const & src_charset,
     }
 }
 
-void
-system_to_utf8(external const & ext, utf8 & utf)
-{
-  string out;
-  charset_convert(system_charset(), "UTF-8", ext(), out);
-  I(utf8_validate(out));
-  utf = out;
-}
-
 size_t
 display_width(utf8 const & utf)
 {
@@ -155,7 +146,7 @@ is_all_ascii(string const & utf)
 
 // this function must be fast.  do not make it slow.
 void
-utf8_to_system(utf8 const & utf, string & ext)
+utf8_to_system_strict(utf8 const & utf, string & ext)
 {
   if (system_charset_is_utf8())
     ext = utf();
@@ -163,15 +154,53 @@ utf8_to_system(utf8 const & utf, string & ext)
            && is_all_ascii(utf()))
     ext = utf();
   else
-    charset_convert("UTF-8", system_charset(), utf(), ext);
+    charset_convert("UTF-8", system_charset(), utf(), ext, false);
+}
+
+// this function must be fast.  do not make it slow.
+void
+utf8_to_system_best_effort(utf8 const & utf, string & ext)
+{
+  if (system_charset_is_utf8())
+    ext = utf();
+  else if (system_charset_is_ascii_extension()
+           && is_all_ascii(utf()))
+    ext = utf();
+  else
+    charset_convert("UTF-8", system_charset(), utf(), ext, true);
 }
 
 void
-utf8_to_system(utf8 const & utf, external & ext)
+utf8_to_system_strict(utf8 const & utf, external & ext)
 {
   string out;
-  utf8_to_system(utf, out);
-  ext = out;
+  utf8_to_system_strict(utf, out);
+  ext = external(out);
+}
+
+void
+utf8_to_system_best_effort(utf8 const & utf, external & ext)
+{
+  string out;
+  utf8_to_system_best_effort(utf, out);
+  ext = external(out);
+}
+
+void
+system_to_utf8(external const & ext, utf8 & utf)
+{
+  if (system_charset_is_utf8())
+    utf = utf8(ext());
+  else if (system_charset_is_ascii_extension()
+           && is_all_ascii(ext()))
+    utf = utf8(ext());
+  else
+    {
+      string out;
+      charset_convert(system_charset(), "UTF-8", ext(), out, false);
+      utf = utf8(out);
+      I(utf8_validate(utf));
+    }
 }
 
 // utf8_validate and the helper functions is_valid_unicode_char and
@@ -287,7 +316,7 @@ ace_to_utf8(ace const & a, utf8 & utf)
     F("error converting %d UTF-8 bytes to IDNA ACE: %s")
     % a().size()
     % decode_idna_error(res));
-  utf = string(out);
+  utf = utf8(string(out));
   free(out);
 }
 
@@ -301,7 +330,7 @@ utf8_to_ace(utf8 const & utf, ace & a)
     F("error converting %d UTF-8 bytes to IDNA ACE: %s")
     % utf().size()
     % decode_idna_error(res));
-  a = string(out);
+  a = ace(string(out));
   free(out);
 }
 
@@ -310,31 +339,17 @@ internalize_cert_name(utf8 const & utf, cert_name & c)
 {
   ace a;
   utf8_to_ace(utf, a);
-  c = a();
+  c = cert_name(a());
 }
 
 void
 internalize_cert_name(external const & ext, cert_name & c)
 {
   utf8 utf;
-  system_to_utf8(ext(), utf);
+  system_to_utf8(ext, utf);
   internalize_cert_name(utf, c);
 }
 
-void
-externalize_cert_name(cert_name const & c, utf8 & utf)
-{
-  ace_to_utf8(ace(c()), utf);
-}
-
-void
-externalize_cert_name(cert_name const & c, external & ext)
-{
-  utf8 utf;
-  externalize_cert_name(c, utf);
-  utf8_to_system(utf, ext);
-}
-
 void
 internalize_rsa_keypair_id(utf8 const & utf, rsa_keypair_id & key)
 {
@@ -351,13 +366,13 @@ internalize_rsa_keypair_id(utf8 const & utf, rsa_keypair_id & key)
       else
         {
           ace a;
-          utf8_to_ace(*i, a);
+          utf8_to_ace(utf8(*i), a);
           tmp += a();
         }
       if (*i == "@")
         in_domain = true;
     }
-  key = tmp;
+  key = rsa_keypair_id(tmp);
 }
 
 void
@@ -391,7 +406,7 @@ externalize_rsa_keypair_id(rsa_keypair_id const & key, utf8 & utf)
       if (*i == "@")
         in_domain = true;
     }
-  utf = tmp;
+  utf = utf8(tmp);
 }
 
 void
@@ -399,7 +414,7 @@ externalize_rsa_keypair_id(rsa_keypair_id const & key, external & ext)
 {
   utf8 utf;
   externalize_rsa_keypair_id(key, utf);
-  utf8_to_system(utf, ext);
+  utf8_to_system_strict(utf, ext);
 }
 
 void
@@ -407,14 +422,14 @@ internalize_var_domain(utf8 const & utf, var_domain & d)
 {
   ace a;
   utf8_to_ace(utf, a);
-  d = a();
+  d = var_domain(a());
 }
 
 void
 internalize_var_domain(external const & ext, var_domain & d)
 {
   utf8 utf;
-  system_to_utf8(ext(), utf);
+  system_to_utf8(ext, utf);
   internalize_var_domain(utf, d);
 }
 
@@ -429,7 +444,7 @@ externalize_var_domain(var_domain const & d, external & ext)
 {
   utf8 utf;
   externalize_var_domain(d, utf);
-  utf8_to_system(utf, ext);
+  utf8_to_system_strict(utf, ext);
 }
 
 
@@ -611,11 +626,11 @@ UNIT_TEST(charset, idna_encoding)
       char *uc = stringprep_ucs4_to_utf8(idna_vec[i].in,
                                          idna_vec[i].inlen,
                                          &p, &q);
-      utf8 utf = string(uc);
+      utf8 utf = utf8(uc);
       utf8 tutf;
       free(uc);
 
-      ace a = string(idna_vec[i].out);
+      ace a = ace(idna_vec[i].out);
       ace tace;
       utf8_to_ace(utf, tace);
       L(FL("ACE-encoded %s: '%s'") % idna_vec[i].name % tace());
@@ -842,10 +857,10 @@ UNIT_TEST(charset, utf8_validation)
   };
 
   for (int i = 0; good_strings[i]; ++i)
-    BOOST_CHECK(utf8_validate(string(good_strings[i])) == true);
+    BOOST_CHECK(utf8_validate(utf8(good_strings[i])) == true);
 
   for (int i = 0; bad_strings[i]; ++i)
-    BOOST_CHECK(utf8_validate(string(bad_strings[i])) == false);
+    BOOST_CHECK(utf8_validate(utf8(bad_strings[i])) == false);
 }
 
 #endif // BUILD_UNIT_TESTS
