@@ -7,8 +7,9 @@
 // implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 // PURPOSE.
 
-#include <iostream>
 #include <cstring>
+#include <iostream>
+#include <iomanip>
 
 #include "cmd.hh"
 #include "diff_patch.hh"
@@ -23,6 +24,7 @@
 #include "safe_map.hh"
 
 using std::cout;
+using std::make_pair;
 using std::map;
 using std::set;
 using std::string;
@@ -69,7 +71,60 @@ three_way_merge(roster_t const & ancestor_roster,
                right_roster, right_markings, right_uncommon_ancestors,
                result);
 }
-  
+
+static bool
+pick_branch_for_update(revision_id chosen_rid, app_state & app)
+{
+  bool switched_branch = false;
+
+  // figure out which branches the target is in
+  vector< revision<cert> > certs;
+  app.db.get_revision_certs(chosen_rid, branch_cert_name, certs);
+  erase_bogus_certs(certs, app);
+
+  set< branch_name > branches;
+  for (vector< revision<cert> >::const_iterator i = certs.begin();
+       i != certs.end(); i++)
+    {
+      cert_value b;
+      decode_base64(i->inner().value, b);
+      branches.insert(branch_name(b()));
+    }
+
+  if (branches.find(app.opts.branchname) != branches.end())
+    {
+      L(FL("using existing branch %s") % app.opts.branchname());
+    }
+  else
+    {
+      P(F("target revision is not in current branch"));
+      if (branches.size() > 1)
+        {
+          // multiple non-matching branchnames
+          string branch_list;
+          for (set<branch_name>::const_iterator i = branches.begin();
+               i != branches.end(); i++)
+            branch_list += "\n  " + (*i)();
+          N(false, F("target revision is in multiple branches:%s\n\n"
+                     "try again with explicit --branch") % branch_list);
+        }
+      else if (branches.size() == 1)
+        {
+          // one non-matching, inform and update
+          app.opts.branchname = *(branches.begin());
+          switched_branch = true;
+        }
+      else
+        {
+          I(branches.size() == 0);
+          W(F("target revision not in any branch\n"
+              "next commit will use branch %s")
+            % app.opts.branchname);
+        }
+    }
+  return switched_branch;
+}
+
 CMD(update, N_("workspace"), "",
     N_("update workspace.\n"
        "This command modifies your workspace to be based off of a\n"
@@ -87,10 +142,13 @@ CMD(update, N_("workspace"), "",
   app.require_workspace();
 
   // Figure out where we are
+  parent_map parents;
+  app.work.get_parent_rosters(parents);
 
-  revision_id old_rid;
-  app.work.get_revision_id(old_rid);
+  N(parents.size() == 1,
+    F("this command can only be used in a single-parent workspace"));
 
+  revision_id old_rid = parent_id(parents.begin());
   N(!null_id(old_rid),
     F("this workspace is a new project; cannot update"));
 
@@ -99,14 +157,14 @@ CMD(update, N_("workspace"), "",
   revision_id chosen_rid;
   if (app.opts.revision_selectors.size() == 0)
     {
-      P(F("updating along branch '%s'") % app.opts.branch_name);
+      P(F("updating along branch '%s'") % app.opts.branchname);
       set<revision_id> candidates;
       pick_update_candidates(old_rid, app, candidates);
       N(!candidates.empty(),
         F("your request matches no descendents of the current revision\n"
           "in fact, it doesn't even match the current revision\n"
           "maybe you want something like --revision=h:%s")
-        % app.opts.branch_name);
+        % app.opts.branchname);
       if (candidates.size() != 1)
         {
           P(F("multiple update candidates:"));
@@ -136,7 +194,7 @@ CMD(update, N_("workspace"), "",
       P(F("already up to date at %s") % old_rid);
       // do still switch the workspace branch, in case they have used
       // update to switch branches.
-      if (!app.opts.branch_name().empty())
+      if (!app.opts.branchname().empty())
         app.make_branch_sticky();
       return;
     }
@@ -145,57 +203,9 @@ CMD(update, N_("workspace"), "",
 
   // Fiddle around with branches, in an attempt to guess what the user
   // wants.
-  
-  bool switched_branch = false;
-  {
-    // figure out which branches the target is in
-    vector< revision<cert> > certs;
-    app.db.get_revision_certs(chosen_rid, branch_cert_name, certs);
-    erase_bogus_certs(certs, app);
-
-    set< utf8 > branches;
-    for (vector< revision<cert> >::const_iterator i = certs.begin();
-         i != certs.end(); i++)
-      {
-        cert_value b;
-        decode_base64(i->inner().value, b);
-        branches.insert(utf8(b()));
-      }
-
-    if (branches.find(app.opts.branch_name) != branches.end())
-      {
-        L(FL("using existing branch %s") % app.opts.branch_name());
-      }
-    else
-      {
-        P(F("target revision is not in current branch"));
-        if (branches.size() > 1)
-          {
-            // multiple non-matching branchnames
-            string branch_list;
-            for (set<utf8>::const_iterator i = branches.begin();
-                 i != branches.end(); i++)
-              branch_list += "\n  " + (*i)();
-            N(false, F("target revision is in multiple branches:%s\n\n"
-                       "try again with explicit --branch") % branch_list);
-          }
-        else if (branches.size() == 1)
-          {
-            // one non-matching, inform and update
-            app.opts.branch_name = (*(branches.begin()))();
-            switched_branch = true;
-            P(F("switching to branch %s") % app.opts.branch_name());
-          }
-        else
-          {
-            I(branches.size() == 0);
-            W(F("target revision not in any branch\n"
-                "next commit will use branch %s")
-              % app.opts.branch_name());
-          }
-      }
-  }
-
+  bool switched_branch = pick_branch_for_update(chosen_rid, app);
+  if (switched_branch)
+    P(F("switching to branch %s") % app.opts.branchname());
 
   // Okay, we have a target, we have a branch, let's do this merge!
 
@@ -218,10 +228,11 @@ CMD(update, N_("workspace"), "",
   temp_node_id_source nis;
 
   // Get the OLD and WORKING rosters
-  shared_ptr<roster_t> old_roster = shared_ptr<roster_t>(new roster_t());
+  database::roster_t_cp old_roster
+    = parent_cached_roster(parents.begin()).first;
   MM(*old_roster);
   roster_t working_roster; MM(working_roster);
-  app.work.get_base_and_current_roster_shape(*old_roster, working_roster, nis);
+  app.work.get_current_roster_shape(working_roster, nis);
   app.work.update_current_roster_from_filesystem(working_roster);
 
   // Get the CHOSEN roster
@@ -234,7 +245,10 @@ CMD(update, N_("workspace"), "",
 
   roster_t & merged_roster = result.roster;
 
-  content_merge_workspace_adaptor wca(app, old_roster);
+  map<file_id, file_path> paths;
+  get_content_paths(working_roster, paths);
+
+  content_merge_workspace_adaptor wca(app, old_roster, paths);
   resolve_merge_conflicts(working_roster, chosen_roster,
                           result, wca, app);
 
@@ -253,17 +267,14 @@ CMD(update, N_("workspace"), "",
 
   // small race condition here...
   app.work.put_work_rev(remaining);
-
-  if (!app.opts.branch_name().empty())
-    {
-      app.make_branch_sticky();
-    }
-  if (switched_branch)
-    P(F("switched branch; next commit will use branch %s") % app.opts.branch_name());
-  P(F("updated to base revision %s") % chosen_rid);
-
   app.work.update_any_attrs();
   app.work.maybe_update_inodeprints();
+
+  if (!app.opts.branchname().empty())
+    app.make_branch_sticky();
+  if (switched_branch)
+    P(F("switched branch; next commit will use branch %s") % app.opts.branchname());
+  P(F("updated to base revision %s") % chosen_rid);
 }
 
 // Subroutine of CMD(merge) and CMD(explicit_merge).  Merge LEFT with RIGHT,
@@ -271,7 +282,7 @@ CMD(update, N_("workspace"), "",
 // bomb out, and therefore so may this.
 static void
 merge_two(revision_id const & left, revision_id const & right,
-          string const & branch, string const & caller, app_state & app)
+          branch_name const & branch, string const & caller, app_state & app)
 {
   // The following mess constructs a neatly formatted log message that looks
   // like this:
@@ -287,14 +298,14 @@ merge_two(revision_id const & left, revision_id const & right,
   ostringstream log;
   size_t fieldwidth = max(caller.size() + strlen(" of '"), strlen("and '"));
 
-  if (branch != app.opts.branch_name())
+  if (branch != app.opts.branchname)
     fieldwidth = max(fieldwidth, strlen("to branch '"));
 
   log << setw(fieldwidth - strlen(" of '")) << caller << " of '" << left
       << "'\n" << setw(fieldwidth) << "and '" << right
       << "'\n";
 
-  if (branch != app.opts.branch_name())
+  if (branch != app.opts.branchname)
     log << setw(fieldwidth) << "to branch '" << branch << "'\n";
 
   // Now it's time for the real work.
@@ -306,8 +317,10 @@ merge_two(revision_id const & left, revision_id const & right,
   interactive_merge_and_store(left, right, merged, app);
 
   packet_db_writer dbw(app);
-  cert_revision_in_branch(merged, branch, app, dbw);
-  cert_revision_changelog(merged, log.str(), app, dbw);
+  app.get_project().put_standard_certs_from_options(merged,
+                                                    branch,
+                                                    utf8(log.str()),
+                                                    dbw);
 
   guard.commit();
   P(F("[merged] %s") % merged);
@@ -326,21 +339,21 @@ CMD(merge, N_("tree"), "", N_("merge unmerged heads of branch"),
   if (args.size() != 0)
     throw usage(name);
 
-  N(app.opts.branch_name() != "",
+  N(app.opts.branchname() != "",
     F("please specify a branch, with --branch=BRANCH"));
 
   set<revision_id> heads;
-  get_branch_heads(app.opts.branch_name(), app, heads);
+  app.get_project().get_branch_heads(app.opts.branchname, heads);
 
-  N(heads.size() != 0, F("branch '%s' is empty") % app.opts.branch_name);
+  N(heads.size() != 0, F("branch '%s' is empty") % app.opts.branchname);
   if (heads.size() == 1)
     {
-      P(F("branch '%s' is already merged") % app.opts.branch_name);
+      P(F("branch '%s' is already merged") % app.opts.branchname);
       return;
     }
 
   P(FP("%d head on branch '%s'", "%d heads on branch '%s'", heads.size())
-    % heads.size() % app.opts.branch_name);
+    % heads.size() % app.opts.branchname);
 
   map<revision_id, revpair> heads_for_ancestor;
   set<revision_id> ancestors;
@@ -394,11 +407,11 @@ CMD(merge, N_("tree"), "", N_("merge unmerged heads of branch"),
       // corresponding pair of heads.
       revpair p = heads_for_ancestor[*ancestors.begin()];
       
-      merge_two(p.first, p.second, app.opts.branch_name(), string("merge"), app);
+      merge_two(p.first, p.second, app.opts.branchname, string("merge"), app);
 
       ancestors.clear();
       heads_for_ancestor.clear();
-      get_branch_heads(app.opts.branch_name(), app, heads);
+      app.get_project().get_branch_heads(app.opts.branchname, heads);
       pass++;
     }
 
@@ -412,7 +425,7 @@ CMD(merge, N_("tree"), "", N_("merge unmerged heads of branch"),
   revision_id right = *i++;
   I(i == heads.end());
   
-  merge_two(left, right, app.opts.branch_name(), string("merge"), app);
+  merge_two(left, right, app.opts.branchname, string("merge"), app);
   P(F("note: your workspaces have not been updated"));
 }
 
@@ -463,8 +476,8 @@ CMD(merge_into_dir, N_("tree"), N_("SOURCE-BRANCH DEST-BRANCH DIR"),
   if (args.size() != 3)
     throw usage(name);
 
-  get_branch_heads(idx(args, 0)(), app, src_heads);
-  get_branch_heads(idx(args, 1)(), app, dst_heads);
+  app.get_project().get_branch_heads(branch_name(idx(args, 0)()), src_heads);
+  app.get_project().get_branch_heads(branch_name(idx(args, 1)()), dst_heads);
 
   N(src_heads.size() != 0, F("branch '%s' is empty") % idx(args, 0)());
   N(src_heads.size() == 1, F("branch '%s' is not merged") % idx(args, 0)());
@@ -492,7 +505,9 @@ CMD(merge_into_dir, N_("tree"), N_("SOURCE-BRANCH DEST-BRANCH DIR"),
         % (*src_i) % idx(args, 1)());
       transaction_guard guard(app.db);
       packet_db_writer dbw(app);
-      cert_revision_in_branch(*src_i, idx(args, 1)(), app, dbw);
+      app.get_project().put_revision_in_branch(*src_i,
+                                               branch_name(idx(args, 1)()),
+                                               dbw);
       guard.commit();
     }
   else
@@ -565,26 +580,119 @@ CMD(merge_into_dir, N_("tree"), N_("SOURCE-BRANCH DEST-BRANCH DIR"),
                                   app);
       }
 
-      packet_db_writer dbw(app);
-
-      cert_revision_in_branch(merged, idx(args, 1)(), app, dbw);
-
       bool log_message_given;
       utf8 log_message;
       process_commit_message_args(log_message_given, log_message, app);
       if (!log_message_given)
-        log_message = (FL("propagate from branch '%s' (head %s)\n"
-                          "            to branch '%s' (head %s)\n")
-                       % idx(args, 0) % (*src_i)
-                       % idx(args, 1) % (*dst_i)).str();
+        log_message = utf8((FL("propagate from branch '%s' (head %s)\n"
+                               "            to branch '%s' (head %s)\n")
+                            % idx(args, 0) % (*src_i)
+                            % idx(args, 1) % (*dst_i)).str());
 
-      cert_revision_changelog(merged, log_message, app, dbw);
+      packet_db_writer dbw(app);
+      app.get_project().put_standard_certs_from_options(merged,
+                                                        branch_name(idx(args, 1)()),
+                                                        log_message,
+                                                        dbw);
 
       guard.commit();
       P(F("[merged] %s") % merged);
     }
 }
 
+CMD(merge_into_workspace, N_("tree"),
+    N_("OTHER-REVISION"),
+    N_("Merge OTHER-REVISION into the current workspace's base revision, "
+       "and update the current workspace with the result.  There can be no "
+       "pending changes in the current workspace.  Both OTHER-REVISION and "
+       "the workspace's base revision will be recorded as parents on commit.  "
+       "The workspace's selected branch is not changed."),
+    options::opts::none)
+{
+  revision_id left_id, right_id;
+  database::cached_roster left, right;
+  roster_t working_roster;
+
+  if (args.size() != 1)
+    throw usage(name);
+
+  app.require_workspace();
+
+  // Get the current state of the workspace.
+
+  // This command cannot be applied to a workspace with more than one parent
+  // (revs can have no more than two parents).
+  {
+    parent_map parents;
+    app.work.get_parent_rosters(parents);
+    N(parents.size() == 1,
+      F("this command can only be used in a single-parent workspace"));
+
+    temp_node_id_source nis;
+    app.work.get_current_roster_shape(working_roster, nis);
+    app.work.update_current_roster_from_filesystem(working_roster);
+
+    N(parent_roster(parents.begin()) == working_roster,
+      F("'%s' can only be used in a workspace with no pending changes") % name);
+
+    left_id = parent_id(parents.begin());
+    left = parent_cached_roster(parents.begin());
+  }
+
+  complete(app, idx(args, 0)(), right_id);
+  app.db.get_roster(right_id, right);
+  N(!(left_id == right_id), F("workspace is already at revision %s") % left_id);
+
+  set<revision_id> left_uncommon_ancestors, right_uncommon_ancestors;
+  app.db.get_uncommon_ancestors(left_id, right_id,
+                                left_uncommon_ancestors,
+                                right_uncommon_ancestors);
+
+  roster_merge_result merge_result;
+  MM(merge_result);
+  roster_merge(*left.first, *left.second, left_uncommon_ancestors,
+               *right.first, *right.second, right_uncommon_ancestors,
+               merge_result);
+
+  revision_id lca_id;
+  database::cached_roster lca;
+  find_common_ancestor_for_merge(left_id, right_id, lca_id, app);
+  app.db.get_roster(lca_id, lca);
+
+  map<file_id, file_path> paths;
+  get_content_paths(working_roster, paths);
+
+  content_merge_workspace_adaptor wca(app, lca.first, paths);
+  resolve_merge_conflicts(*left.first, *right.first, merge_result, wca, app);
+
+  // Make sure it worked...
+  I(merge_result.is_clean());
+  merge_result.roster.check_sane(true);
+
+  // Construct the workspace revision.
+  parent_map parents;
+  safe_insert(parents, std::make_pair(left_id, left));
+  safe_insert(parents, std::make_pair(right_id, right));
+
+  revision_t merged_rev;
+  make_revision_for_workspace(parents, merge_result.roster, merged_rev);
+
+  // Note: the csets in merged_rev are _not_ suitable for submission to
+  // perform_content_update, because content changes have been dropped.
+  cset update;
+  make_cset(*left.first, merge_result.roster, update);
+
+  // small race condition here...
+  app.work.perform_content_update(update, wca);
+  app.work.put_work_rev(merged_rev);
+  app.work.update_any_attrs();
+  app.work.maybe_update_inodeprints();
+
+  P(F("updated to result of merge\n"
+      " [left] %s\n"
+      "[right] %s\n") % left_id % right_id);
+}
+
 CMD(explicit_merge, N_("tree"),
     N_("LEFT-REVISION RIGHT-REVISION DEST-BRANCH"),
     N_("merge two explicitly given revisions, "
@@ -592,14 +700,14 @@ CMD(explicit_merge, N_("tree"),
     options::opts::date | options::opts::author)
 {
   revision_id left, right;
-  string branch;
+  branch_name branch;
 
   if (args.size() != 3)
     throw usage(name);
 
   complete(app, idx(args, 0)(), left);
   complete(app, idx(args, 1)(), right);
-  branch = idx(args, 2)();
+  branch = branch_name(idx(args, 2)());
 
   N(!(left == right),
     F("%s and %s are the same revision, aborting") % left % right);
@@ -650,7 +758,7 @@ CMD(show_conflicts, N_("informative"), N_("REV REV"),
     % result.rename_target_conflicts.size());
   P(F("There are %s directory_loop_conflicts.") 
     % result.directory_loop_conflicts.size());
-}                                                                
+}
 
 CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
     N_("Apply changes made at arbitrary places in history to current workspace.\n"
@@ -661,7 +769,7 @@ CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
        "renames, conflicts, and so on.\n"
        "\n"
        "If one revision is given, applies the changes made in that revision\n"
-       "compared to its parent.\n"                                                                                  
+       "compared to its parent.\n"
        "\n"
        "If two revisions are given, applies the changes made to get from the\n"  
        "first revision to the second."),
@@ -735,10 +843,10 @@ CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
   MM(*from_roster);
   app.db.get_roster(from_rid, *from_roster);
 
-  // Get the WORKING roster, and also the base roster while we're at it
+  // Get the WORKING roster
   roster_t working_roster; MM(working_roster);
-  roster_t base_roster; MM(base_roster);
-  app.work.get_base_and_current_roster_shape(base_roster, working_roster, nis);
+  app.work.get_current_roster_shape(working_roster, nis);
+
   app.work.update_current_roster_from_filesystem(working_roster);
 
   // Get the FROM->TO cset...
@@ -771,7 +879,10 @@ CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
 
   roster_t & merged_roster = result.roster;
 
-  content_merge_workspace_adaptor wca(app, from_roster);
+  map<file_id, file_path> paths;
+  get_content_paths(working_roster, paths);
+
+  content_merge_workspace_adaptor wca(app, from_roster, paths);
   resolve_merge_conflicts(working_roster, to_roster,
                           result, wca, app);
 
@@ -789,11 +900,11 @@ CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
   P(F("applied changes to workspace"));
 
   // and record any remaining changes in _MTN/revision
-  revision_id base_id;
+  parent_map parents;
   revision_t remaining;
   MM(remaining);
-  app.work.get_revision_id(base_id);
-  make_revision_for_workspace(base_id, base_roster, merged_roster, remaining);
+  app.work.get_parent_rosters(parents);
+  make_revision_for_workspace(parents, merged_roster, remaining);
 
   // small race condition here...
   app.work.put_work_rev(remaining);
@@ -825,40 +936,87 @@ CMD(heads, N_("tree"), "", N_("show unmerged head revisions of branch"),
   if (args.size() != 0)
     throw usage(name);
 
-  N(app.opts.branch_name() != "",
+  N(app.opts.branchname() != "",
     F("please specify a branch, with --branch=BRANCH"));
 
-  get_branch_heads(app.opts.branch_name(), app, heads);
+  app.get_project().get_branch_heads(app.opts.branchname, heads);
 
   if (heads.size() == 0)
-    P(F("branch '%s' is empty") % app.opts.branch_name);
+    P(F("branch '%s' is empty") % app.opts.branchname);
   else if (heads.size() == 1)
-    P(F("branch '%s' is currently merged:") % app.opts.branch_name);
+    P(F("branch '%s' is currently merged:") % app.opts.branchname);
   else
-    P(F("branch '%s' is currently unmerged:") % app.opts.branch_name);
+    P(F("branch '%s' is currently unmerged:") % app.opts.branchname);
 
   for (set<revision_id>::const_iterator i = heads.begin();
        i != heads.end(); ++i)
-    cout << describe_revision(app, *i) << "\n";
+    cout << describe_revision(app, *i) << '\n';
 }
 
-CMD(get_roster, N_("debug"), N_("REVID"),
-    N_("dump the roster associated with the given REVID"),
+CMD(get_roster, N_("debug"), N_("[REVID]"),
+    N_("dump the roster associated with the given REVID, "
+       "or the workspace if no REVID is given"),
     options::opts::none)
 {
-  revision_id rid;
-  if (args.size() == 0)
-    app.work.get_revision_id(rid);
-  else if (args.size() == 1)
-    complete(app, idx(args, 0)(), rid);
-  else
-    throw usage(name);
-
-  I(!null_id(rid));
-
   roster_t roster;
   marking_map mm;
-  app.db.get_roster(rid, roster, mm);
+  
+  if (args.size() == 0)
+    {
+      parent_map parents;
+      temp_node_id_source nis;
+      revision_id rid(fake_id());
+      
+      app.require_workspace();
+      app.work.get_parent_rosters(parents);
+      app.work.get_current_roster_shape(roster, nis);
+      app.work.update_current_roster_from_filesystem(roster);
+
+      if (parents.size() == 0)
+        {
+          mark_roster_with_no_parents(rid, roster, mm);
+        }
+      else if (parents.size() == 1)
+        {
+          roster_t parent = parent_roster(parents.begin());
+          marking_map parent_mm = parent_marking(parents.begin());
+          mark_roster_with_one_parent(parent, parent_mm, rid, roster, mm);
+        }
+      else
+        {
+          parent_map::const_iterator i = parents.begin();
+          revision_id left_id = parent_id(i);
+          roster_t const & left_roster = parent_roster(i);
+          marking_map const & left_markings = parent_marking(i);
+
+          i++;
+          revision_id right_id = parent_id(i);
+          roster_t const & right_roster = parent_roster(i);
+          marking_map const & right_markings = parent_marking(i);
+
+          i++; I(i == parents.end());
+
+          set<revision_id> left_uncommon_ancestors, right_uncommon_ancestors;
+          app.db.get_uncommon_ancestors(left_id, right_id,
+                                        left_uncommon_ancestors,
+                                        right_uncommon_ancestors);
+
+          mark_merge_roster(left_roster, left_markings,
+                            left_uncommon_ancestors,
+                            right_roster, right_markings,
+                            right_uncommon_ancestors,
+                            rid, roster, mm);
+        }
+    }
+  else if (args.size() == 1)
+    {
+      revision_id rid;
+      complete(app, idx(args, 0)(), rid);
+      I(!null_id(rid));
+      app.db.get_roster(rid, roster, mm);
+    }
+  else
+    throw usage(name);
 
   roster_data dat;
   write_roster_and_marking(roster, mm, dat);
