@@ -541,8 +541,7 @@ Netxx::PipeCompatibleProbe::add(const StreamServer &ss, ready_type rt)
 #ifdef BUILD_UNIT_TESTS
 #include "unit_tests.hh"
 
-static void
-simple_pipe_test()
+UNIT_TEST(pipe, simple_pipe)
 { try
   {
   Netxx::PipeStream pipe("cat",vector<string>());
@@ -603,17 +602,10 @@ catch (informative_failure &e)
   // for some reason boost does not provide
   // enough information
   {
-    W(F("Failure %s") % e.what);
+    W(F("Failure %s") % e.what());
     throw;
   }
 }
-
-void
-add_pipe_tests(test_suite * suite)
-{
-  I(suite);
-  suite->add(BOOST_TEST_CASE(&simple_pipe_test));
-}
 #endif
 
 // Local Variables:
