@@ -8,7 +8,6 @@
 // PURPOSE.
 
 #include <algorithm>
-#include <iostream>
 #include <iterator>
 #include <list>
 #include <map>
@@ -20,7 +19,6 @@
 #include <vector>
 
 #include <unistd.h>
-#include <cstdio>
 
 #include <boost/shared_ptr.hpp>
 #include <boost/scoped_ptr.hpp>
@@ -38,6 +36,7 @@
 #include "packet.hh"
 #include "paths.hh"
 #include "platform-wrapped.hh"
+#include "project.hh"
 #include "rcs_file.hh"
 #include "revision.hh"
 #include "safe_map.hh"
@@ -464,7 +463,7 @@ rcs_put_raw_file_edge(hexenc<id> const & old_id,
       return;
     }
 
-  if (db.file_version_exists(old_id))
+  if (db.file_version_exists(file_id(old_id)))
     {
       // we already have a way to get to this old version,
       // no need to insert another reconstruction path
@@ -472,7 +471,7 @@ rcs_put_raw_file_edge(hexenc<id> const & old_id,
     }
   else
     {
-      I(db.file_or_manifest_base_exists(new_id(), "files")
+      I(db.file_or_manifest_base_exists(new_id, "files")
         || db.delta_exists(new_id(), "file_deltas"));
       db.put_file_delta(file_id(old_id), file_id(new_id), file_delta(del));
     }
@@ -494,7 +493,7 @@ insert_into_db(data const & curr_data,
   {
     string tmp;
     global_pieces.build_string(next_lines, tmp);
-    next_data = tmp;
+    next_data = data(tmp);
   }
   delta del;
   diff(curr_data, next_data, del);
@@ -582,7 +581,7 @@ process_branch(string const & begin_version,
     {
       L(FL("version %s has %d lines") % curr_version % curr_lines->size());
 
-      cvs_commit curr_commit(r, curr_version, curr_id, cvs);
+      cvs_commit curr_commit(r, curr_version, file_id(curr_id), cvs);
       if (!curr_commit.is_synthetic_branch_root)
         {
           cvs.stk.top()->append_commit(curr_commit);
@@ -682,14 +681,14 @@ import_rcs_file_with_cvs(string const & filename, database & db, cvs_history & c
     hexenc<id> id;
     data dat(r.deltatexts.find(r.admin.head)->second->text);
     calculate_ident(dat, id);
-    file_id fid = id;
+    file_id fid(id);
 
     cvs.set_filename (filename, fid);
     cvs.index_branchpoint_symbols (r);
 
     if (! db.file_version_exists (fid))
       {
-        db.put_file(fid, dat);
+        db.put_file(fid, file_data(dat));
       }
 
     {
@@ -1219,8 +1218,8 @@ import_cvs_repo(system_path const & cvsroot,
   }
 
   cvs_history cvs;
-  N(app.opts.branch_name() != "", F("need base --branch argument for importing"));
-  cvs.base_branch = app.opts.branch_name();
+  N(app.opts.branchname() != "", F("need base --branch argument for importing"));
+  cvs.base_branch = app.opts.branchname();
 
   // push the trunk
   cvs.trunk = shared_ptr<cvs_branch>(new cvs_branch());
@@ -1274,7 +1273,7 @@ import_cvs_repo(system_path const & cvsroot,
       {
         string tag = cvs.tag_interner.lookup(i->first);
         ui.set_tick_trailer("marking tag " + tag);
-        cert_revision_tag(i->second.second, tag, app, dbw);
+        app.get_project().put_tag(i->second.second, tag, dbw);
         ++n_tags;
       }
     guard.commit();
@@ -1386,10 +1385,12 @@ cluster_consumer::store_auxiliary_certs(prepared_revision const & p)
         }
     }
 
-  cert_revision_in_branch(p.rid, cert_value(branchname), app, dbw);
-  cert_revision_author(p.rid, cvs.author_interner.lookup(p.author), app, dbw);
-  cert_revision_changelog(p.rid, cvs.changelog_interner.lookup(p.changelog), app, dbw);
-  cert_revision_date_time(p.rid, p.time, app, dbw);
+  app.get_project().put_standard_certs(p.rid,
+                                       branch_name(branchname),
+                                       utf8(cvs.changelog_interner.lookup(p.changelog)),
+                                       date_t::from_unix_epoch(p.time),
+                                       utf8(cvs.author_interner.lookup(p.author)),
+                                       dbw);
 }
 
 void
