@@ -9,7 +9,6 @@
 
 #include <cctype>
 #include <cstdlib>
-#include <iostream>
 #include <map>
 #include <queue>
 #include <set>
@@ -42,6 +41,8 @@
 #include "vocab.hh"
 #include "safe_map.hh"
 #include "legacy.hh"
+#include "rev_height.hh"
+#include "cmd.hh"
 
 using std::back_inserter;
 using std::copy;
@@ -66,6 +67,10 @@ void revision_t::check_sane() const
 {
   // null id in current manifest only permitted if previous
   // state was null and no changes
+  // FIXME: above comment makes no sense.  This should just be
+  // I(!null_id(new_manifest)), and the only reason I am not making it so
+  // right now is that I don't have time to immediately track down all the
+  // fallout.
   if (null_id(new_manifest))
     {
       for (edge_map::const_iterator i = edges.begin();
@@ -383,53 +388,27 @@ calculate_ancestors_from_graph(interner<ctx> & intern,
     }
 }
 
-// this function actually toposorts the whole graph, and then filters by the
-// passed in set.  if anyone ever needs to toposort the whole graph, then,
-// this function would be a good thing to generalize...
 void
 toposort(set<revision_id> const & revisions,
          vector<revision_id> & sorted,
          app_state & app)
 {
-  sorted.clear();
-  typedef multimap<revision_id, revision_id>::iterator gi;
-  typedef map<revision_id, int>::iterator pi;
-  multimap<revision_id, revision_id> graph;
-  app.db.get_revision_ancestry(graph);
-  set<revision_id> leaves;
-  app.db.get_revision_ids(leaves);
-  map<revision_id, int> pcount;
-  for (gi i = graph.begin(); i != graph.end(); ++i)
-    pcount.insert(make_pair(i->first, 0));
-  for (gi i = graph.begin(); i != graph.end(); ++i)
-    ++(pcount[i->second]);
-  // first find the set of graph roots
-  list<revision_id> roots;
-  for (pi i = pcount.begin(); i != pcount.end(); ++i)
-    if(i->second==0)
-      roots.push_back(i->first);
-  while (!roots.empty())
+  map<rev_height, revision_id> work;
+
+  for (set<revision_id>::const_iterator i = revisions.begin();
+       i != revisions.end(); ++i) 
     {
-      // now stick them in our ordering (if wanted) and remove them from the
-      // graph, calculating the new roots as we go
-      L(FL("new root: %s") % (roots.front()));
-      if (revisions.find(roots.front()) != revisions.end())
-        sorted.push_back(roots.front());
-      for(gi i = graph.lower_bound(roots.front());
-          i != graph.upper_bound(roots.front()); i++)
-        if(--(pcount[i->second]) == 0)
-          roots.push_back(i->second);
-      graph.erase(roots.front());
-      leaves.erase(roots.front());
-      roots.pop_front();
+      rev_height height;
+      app.db.get_rev_height(*i, height);
+      work.insert(make_pair(height, *i));
     }
-  I(graph.empty());
-  for (set<revision_id>::const_iterator i = leaves.begin();
-       i != leaves.end(); ++i)
+
+  sorted.clear();
+  
+  for (map<rev_height, revision_id>::const_iterator i = work.begin();
+       i != work.end(); ++i)
     {
-      L(FL("new leaf: %s") % (*i));
-      if (revisions.find(*i) != revisions.end())
-        sorted.push_back(*i);
+      sorted.push_back(i->second);
     }
 }
 
@@ -671,6 +650,99 @@ make_revision(revision_id const & old_rev_id,
   rev.made_for = made_for_database;
 }
 
+void
+make_revision(parent_map const & old_rosters,
+              roster_t const & new_roster,
+              revision_t & rev)
+{
+  edge_map edges;
+  for (parent_map::const_iterator i = old_rosters.begin();
+       i != old_rosters.end();
+       i++)
+    {
+      shared_ptr<cset> cs(new cset());
+      make_cset(parent_roster(i), new_roster, *cs);
+      safe_insert(edges, make_pair(parent_id(i), cs));
+    }
+
+  rev.edges = edges;
+  calculate_ident(new_roster, rev.new_manifest);
+  L(FL("new manifest_id is %s") % rev.new_manifest);
+}
+
+static void
+recalculate_manifest_id_for_restricted_rev(parent_map const & old_rosters,
+                                           edge_map & edges,
+                                           revision_t & rev)
+{
+  // In order to get the correct manifest ID, recalculate the new roster
+  // using one of the restricted csets.  It doesn't matter which of the
+  // parent roster/cset pairs we use for this; by construction, they must
+  // all produce the same result.
+  revision_id id = parent_id(old_rosters.begin());
+  roster_t restricted_roster = *(safe_get(old_rosters, id).first);
+
+  temp_node_id_source nis;
+  editable_roster_base er(restricted_roster, nis);
+  safe_get(edges, id)->apply_to(er);
+
+  calculate_ident(restricted_roster, rev.new_manifest);
+  rev.edges = edges;
+  L(FL("new manifest_id is %s") % rev.new_manifest);
+}
+
+void
+make_restricted_revision(parent_map const & old_rosters,
+                         roster_t const & new_roster,
+                         node_restriction const & mask,
+                         revision_t & rev)
+{
+  edge_map edges;
+  cset dummy;
+  for (parent_map::const_iterator i = old_rosters.begin();
+       i != old_rosters.end();
+       i++)
+    {
+      shared_ptr<cset> included(new cset());
+      make_restricted_csets(parent_roster(i), new_roster,
+                            *included, dummy, mask);
+      check_restricted_cset(parent_roster(i), *included);
+      safe_insert(edges, make_pair(parent_id(i), included));
+    }
+
+  recalculate_manifest_id_for_restricted_rev(old_rosters, edges, rev);
+}
+
+void
+make_restricted_revision(parent_map const & old_rosters,
+                         roster_t const & new_roster,
+                         node_restriction const & mask,
+                         revision_t & rev,
+                         cset & excluded,
+                         string const & cmd_name)
+{
+  edge_map edges;
+  bool no_excludes = true;
+  for (parent_map::const_iterator i = old_rosters.begin();
+       i != old_rosters.end();
+       i++)
+    {
+      shared_ptr<cset> included(new cset());
+      make_restricted_csets(parent_roster(i), new_roster,
+                            *included, excluded, mask);
+      check_restricted_cset(parent_roster(i), *included);
+      safe_insert(edges, make_pair(parent_id(i), included));
+      if (!excluded.empty())
+        no_excludes = false;
+    }
+
+  N(old_rosters.size() == 1 || no_excludes,
+    F("the command '%s %s' cannot be restricted in a two-parent workspace")
+    % ui.prog_name % cmd_name);
+
+  recalculate_manifest_id_for_restricted_rev(old_rosters, edges, rev);
+}
+
 // Workspace-only revisions, with fake rev.new_manifest and content
 // changes suppressed.
 void
@@ -687,7 +759,7 @@ make_revision_for_workspace(revision_id const & old_rev_id,
   rev.edges.clear();
   safe_insert(rev.edges, make_pair(old_rev_id, cs));
   if (!null_id(old_rev_id))
-    rev.new_manifest = fake_id();
+    rev.new_manifest = manifest_id(fake_id());
   rev.made_for = made_for_workspace;
 }
 
@@ -706,6 +778,28 @@ make_revision_for_workspace(revision_id const & old_rev_id,
   make_revision_for_workspace(old_rev_id, changes, rev);
 }
 
+void
+make_revision_for_workspace(parent_map const & old_rosters,
+                            roster_t const & new_roster,
+                            revision_t & rev)
+{
+  edge_map edges;
+  for (parent_map::const_iterator i = old_rosters.begin();
+       i != old_rosters.end();
+       i++)
+    {
+      shared_ptr<cset> cs(new cset());
+      make_cset(parent_roster(i), new_roster, *cs);
+      cs->deltas_applied.clear();
+      safe_insert(edges, make_pair(parent_id(i), cs));
+    }
+
+  rev.edges = edges;
+  rev.new_manifest = manifest_id(fake_id());
+  rev.made_for = made_for_workspace;
+}
+
+
 // Stuff related to rebuilding the revision graph. Unfortunately this is a
 // real enough error case that we need support code for it.
 
@@ -718,7 +812,7 @@ dump(parent_roster_map const & prm, string & out)
   ostringstream oss;
   for (parent_roster_map::const_iterator i = prm.begin(); i != prm.end(); ++i)
     {
-      oss << "roster: " << i->first << "\n";
+      oss << "roster: " << i->first << '\n';
       string roster_str, indented_roster_str;
       dump(*i->second.first, roster_str);
       prefix_lines_with("    ", roster_str, indented_roster_str);
@@ -809,7 +903,7 @@ void anc_graph::write_certs()
         encode_hexenc(data(string(buf, buf + constants::epochlen_bytes)), hexdata);
         epoch_data new_epoch(hexdata);
         L(FL("setting epoch for %s to %s") % *i % new_epoch);
-        app.db.set_epoch(cert_value(*i), new_epoch);
+        app.db.set_epoch(branch_name(*i), new_epoch);
       }
   }
 
@@ -1630,8 +1724,8 @@ build_changesets_from_manifest_ancestry(app_state & app)
       cert_value tv;
       decode_base64(i->inner().value, tv);
       manifest_id child, parent;
-      child = i->inner().ident;
-      parent = hexenc<id>(tv());
+      child = manifest_id(i->inner().ident);
+      parent = manifest_id(tv());
 
       u64 parent_node = graph.add_node_for_old_manifest(parent);
       u64 child_node = graph.add_node_for_old_manifest(child);
@@ -1641,21 +1735,65 @@ build_changesets_from_manifest_ancestry(app_state & app)
   graph.rebuild_ancestry();
 }
 
-void
-regenerate_rosters(app_state & app)
+
+// This is a special function solely for the use of regenerate_caches -- it
+// must work even when caches (especially, the height cache!) do not exist.
+// For all other purposes, use toposort above.
+static void
+allrevs_toposorted(vector<revision_id> & revisions,
+                   app_state & app)
 {
-  P(F("regenerating cached rosters"));
+
+  typedef multimap<revision_id, revision_id>::const_iterator gi;
+  typedef map<revision_id, int>::iterator pi;
+
+  revisions.clear();
+
+  // get the complete ancestry
+  multimap<revision_id, revision_id> graph;
+  app.db.get_revision_ancestry(graph);
+
+  // determine the number of parents for each rev
+  map<revision_id, int> pcount;
+  for (gi i = graph.begin(); i != graph.end(); ++i)
+    pcount.insert(make_pair(i->first, 0));
+  for (gi i = graph.begin(); i != graph.end(); ++i)
+    ++(pcount[i->second]);
+
+  // find the set of graph roots
+  list<revision_id> roots;
+  for (pi i = pcount.begin(); i != pcount.end(); ++i)
+    if(i->second==0)
+      roots.push_back(i->first);
+
+  while (!roots.empty())
+    {
+      revision_id cur = roots.front();
+      roots.pop_front();
+      if (!null_id(cur))
+        revisions.push_back(cur);
+      
+      for(gi i = graph.lower_bound(cur);
+          i != graph.upper_bound(cur); i++)
+        if(--(pcount[i->second]) == 0)
+          roots.push_back(i->second);
+    }
+}
+
+void
+regenerate_caches(app_state & app)
+{
+  P(F("regenerating cached rosters and heights"));
 
   app.db.ensure_open_for_format_changes();
+
   transaction_guard guard(app.db);
 
   app.db.delete_existing_rosters();
+  app.db.delete_existing_heights();
 
-  std::set<revision_id> ids;
-  app.db.get_revision_ids(ids);
-  P(F("calculating rosters to regenerate"));
-  std::vector<revision_id> sorted_ids;
-  toposort(ids, sorted_ids, app);
+  vector<revision_id> sorted_ids;
+  allrevs_toposorted(sorted_ids, app);
 
   ticker done(_("regenerated"), "r", 5);
   done.set_total(sorted_ids.size());
@@ -1664,12 +1802,28 @@ regenerate_rosters(app_state & app)
        i != sorted_ids.end(); ++i)
     {
       revision_t rev;
-      app.db.get_revision(*i, rev);
-      app.db.put_roster_for_revision(*i, rev);
+      revision_id const & rev_id = *i;
+      app.db.get_revision(rev_id, rev);
+      app.db.put_roster_for_revision(rev_id, rev);
+      app.db.put_height_for_revision(rev_id, rev);
       ++done;
     }
 
   guard.commit();
+
+  P(F("finished regenerating cached rosters and heights"));
+}
+
+CMD(rev_height, hidden_group(), N_("REV"), "show the height for REV",
+    options::opts::none)
+{
+  if (args.size() != 1)
+    throw usage(name);
+  revision_id rid(idx(args, 0)());
+  N(app.db.revision_exists(rid), F("No such revision %s") % rid);
+  rev_height height;
+  app.db.get_rev_height(rid, height);
+  P(F("cached height: %s") % height);
 }
 
 // i/o stuff
@@ -1825,7 +1979,7 @@ void calculate_ident(revision_t const & cs,
   hexenc<id> tid;
   write_revision(cs, tmp);
   calculate_ident(tmp, tid);
-  ident = tid;
+  ident = revision_id(tid);
 }
 
 #ifdef BUILD_UNIT_TESTS
