@@ -70,7 +70,8 @@ tilde_expand(std::string const & in)
       fs::path res;
       if (*i == "~")
         {
-          res /= get_homedir();
+          fs::path restmp(get_homedir(), fs::native);
+          res /= restmp;
           ++i;
         }
       else if (i->size() > 1 && i->at(0) == '~')
@@ -123,3 +124,11 @@ rename_clobberingly(std::string const & from, std::string const & to)
   E(!rename(from.c_str(), to.c_str()),
     F("renaming '%s' to '%s' failed: %s") % from % to % os_strerror(errno));
 }
+
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
