@@ -139,38 +139,45 @@ get_path_status(std::string const & path)
 }
 
 static bool
-rename_clobberingly_impl(const char* from, const char* to)
+rename_clobberingly_impl(const char * from, const char * to)
 {
   // MoveFileEx is only available on NT-based systems.  We will revert to a
   // more compatible DeleteFile/MoveFile pair as a compatibility fall-back.
   typedef BOOL (WINAPI *MoveFileExFun)(LPCTSTR, LPCTSTR, DWORD);
   static MoveFileExFun fnMoveFileEx = 0;
   static bool MoveFileExAvailable = false;
-  if (fnMoveFileEx == 0) {
-    HMODULE hModule = LoadLibrary("kernel32");
-    if (hModule)
-      fnMoveFileEx = reinterpret_cast<MoveFileExFun>
-        (GetProcAddress(hModule, "MoveFileExA"));
-    if (fnMoveFileEx) {
-      L(FL("using MoveFileEx for renames"));
-      MoveFileExAvailable = true;
-    } else
-      L(FL("using DeleteFile/MoveFile fallback for renames"));
-  }
+  if (fnMoveFileEx == 0)
+    {
+      HMODULE hModule = LoadLibrary("kernel32");
+      if (hModule)
+	fnMoveFileEx = reinterpret_cast<MoveFileExFun>
+	  (GetProcAddress(hModule, "MoveFileExA"));
+      if (fnMoveFileEx)
+	{
+	  L(FL("using MoveFileEx for renames"));
+	  MoveFileExAvailable = true;
+	}
+      else
+	L(FL("using DeleteFile/MoveFile fallback for renames"));
+    }
 
-  if (MoveFileExAvailable) {
-    if (fnMoveFileEx(from, to, MOVEFILE_REPLACE_EXISTING))
-      return true;
-    else if (GetLastError() == ERROR_CALL_NOT_IMPLEMENTED) {
-      MoveFileExAvailable = false;
-      L(FL("MoveFileEx failed with CALL_NOT_IMPLEMENTED, using fallback"));
+  if (MoveFileExAvailable)
+    {
+      if (fnMoveFileEx(from, to, MOVEFILE_REPLACE_EXISTING))
+	return true;
+      else if (GetLastError() == ERROR_CALL_NOT_IMPLEMENTED)
+	{
+	  MoveFileExAvailable = false;
+	  L(FL("MoveFileEx failed with CALL_NOT_IMPLEMENTED, using fallback"));
+	}
+    }
+  else
+    {
+      // This is not even remotely atomic, but what can you do?
+      DeleteFile(to);
+      if (MoveFile(from, to))
+	return true;
     }
-  } else {
-    // This is not even remotely atomic, but what can you do?
-    DeleteFile(to);
-    if (MoveFile(from, to))
-      return true;
-  }
   return false;
 }
 
@@ -199,3 +206,10 @@ rename_clobberingly(std::string const & from, std::string const & to)
            % os_strerror(lastError) % lastError);
 }
 
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
