@@ -20,12 +20,20 @@ os_strerror(os_err_t errnum)
                             static_cast<va_list *>(0));
   if (len == 0)
     return (F("unknown error code %d") % errnum).str();
-  // clobber trailing newlines.
-  for (LPTSTR p = tstr + len - 1; p > tstr; --p)
-    if (*p == '\r' || *p == '\n')
-      *p = '\0';
-  std::string str = tstr;
+  std::string errstr = tstr;
   LocalFree(tstr);
-  return str;
+
+  // clobber trailing newlines.
+  std::string::size_type end = errstr.find_last_not_of("\r\n");
+  if (end != std::string::npos)
+    errstr.erase(end + 1);
+  return errstr;
 }
 
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
