@@ -158,6 +158,11 @@ struct redir
 redir::redir(int which, char const * filename)
  : what(which)
 {
+  if (!filename || *filename == '\0')
+    {
+      what = -1;
+      return;
+    }
   HANDLE file;
   SECURITY_ATTRIBUTES sa;
   sa.nLength = sizeof(SECURITY_ATTRIBUTES);
@@ -223,6 +228,12 @@ pid_t process_spawn_redirected(char const * in,
     }
 }
 
+pid_t process_spawn_pipe(char const * const argv[], FILE** in, FILE** out)
+{
+  // XXX: not implemented on Win32 yet
+  return -1;
+}
+
 int process_wait(pid_t pid, int *res, int timeout)
 {
   HANDLE hProcess = (HANDLE)pid;
@@ -262,3 +273,10 @@ pid_t get_process_id()
   return GetCurrentProcessId();
 }
 
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
